/*
 * Copyright 1999 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

/*
 * Name:    mt_scan.c
 *
 * Purpose: Tcl interface to generic MX scan functions.
 */

#include <stdio.h>

#include <tcl.h>
#include <itcl.h>

#include "mxconfig.h"
#include "mx_util.h"
#include "mx_rec.h"
#include "mx_scan.h"
#include "mx_handl.h"
#include "mx_tcl.h"
#include "mx_tclpr.h"

#include "mx_key.h"

MX_EXPORT int
Mx_scan_perform_scan_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	const char fname[] = "Mx_scan_perform_scan_cmd()";

	MX_RECORD *record;
	MX_SCAN *scan;
	mx_status_type mx_status;

	if ( objc != 2 ) {
		Tcl_WrongNumArgs( interp, 1, objv,
		"Usage: scan_perform_scan record_handle" );
		return TCL_ERROR;
	}

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return TCL_ERROR;

	scan = (MX_SCAN *) record->record_superclass_struct;

	if ( scan == (MX_SCAN *) NULL ) {
		(void) mx_error( MXE_CORRUPT_DATA_STRUCTURE, fname,
		"MX_SCAN pointer for record '%s' is NULL.", record->name );

		return TCL_ERROR;
	}

	mx_status = mx_perform_scan( scan );

	if ( mx_status.code != MXE_SUCCESS )
		return TCL_ERROR;

	return TCL_OK;
}

